//
// fraktal - visualization of Mandelbrot and Julia sets
// Copyright (C) 1999-2020  Joachim Reichel <joachim.reichel@posteo.de>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef FRAKTAL_SHADER_MANDELBROT_H
#define FRAKTAL_SHADER_MANDELBROT_H

#include "shader_common.h"

const char* g_mandelbrot_shader_string = " \
\
varying vec4 position; \
uniform float kmax; \
\
float mandelbrot (float a, float b) \
{ \
    float u = 4.0 * (a*a + b*b); \
    float v = u - 2.0*a + 0.25; \
    float c; \
\
    if (u + 8.0*a + 3.75 < 0.0) \
        c = 0.0; \
    else if (v - sqrt(v) + 2.0*a - 0.5 < 0.0) \
        c = 0.0; \
    else \
        { \
\
            float k = 0.0; \
            float x1 = a; \
            float y1 = b; \
            float x2; \
            float y2; \
            float d; \
\
            do \
                { \
                    x2 = x1*x1 - y1*y1 + a; \
                    y2 = 2.0 * x1 * y1 + b; \
                    d  = x2*x2 + y2*y2; \
                    x1 = x2; \
                    y1 = y2; \
                    k++; \
                } \
            while ((d <= 4.0) && (k < kmax)); \
\
            if (k >= kmax) \
                c = 0.0; \
            else \
                c = k + log ( log(2.0*2.0)/(log(d)) ) / log (2.0); \
        } \
\
    return c; \
} \
\
" g_compute_color_string "\
\
void main() \
{ \
    float x = mandelbrot (position.x, position.y); \
    gl_FragColor = compute_color (x); \
} \
";

#endif // FRAKTAL_SHADER_MANDELBROT_H
